@@ -10,38 +10,38 @@ use Carp::Clan qw/^DBIx::Class/;
 use Text::Balanced qw( extract_bracketed );
 use Class::C3;
 
-our $VERSION = '0.06001';
+our $VERSION = '0.07000';
 
 =head1 NAME
 
 DBIx::Class::Schema::Loader::DBI::SQLite - DBIx::Class::Schema::Loader::DBI SQLite Implementation.
 
-=head1 SYNOPSIS
-
-  package My::Schema;
-  use base qw/DBIx::Class::Schema::Loader/;
-
-  __PACKAGE__->loader_options( debug => 1 );
-
-  1;
-
 =head1 DESCRIPTION
 
-See L<DBIx::Class::Schema::Loader::Base>.
+See L<DBIx::Class::Schema::Loader> and L<DBIx::Class::Schema::Loader::Base>.
 
 =head1 METHODS
 
 =head2 rescan
 
-SQLite will fail all further commands on a connection if the
-underlying schema has been modified.  Therefore, any runtime
-changes requiring C<rescan> also require us to re-connect
-to the database.  The C<rescan> method here handles that
-reconnection for you, but beware that this must occur for
-any other open sqlite connections as well.
+SQLite will fail all further commands on a connection if the underlying schema
+has been modified.  Therefore, any runtime changes requiring C<rescan> also
+require us to re-connect to the database.  The C<rescan> method here handles
+that reconnection for you, but beware that this must occur for any other open
+sqlite connections as well.
 
 =cut
 
+sub _setup {
+    my $self = shift;
+
+    $self->next::method(@_);
+
+    if (not defined $self->preserve_case) {
+        $self->preserve_case(0);
+    }
+}
+
 sub rescan {
     my ($self, $schema) = @_;
 
@@ -49,32 +49,50 @@ sub rescan {
     $self->next::method($schema);
 }
 
-sub _extra_column_info {
-    my ($self, $table, $col_name, $info, $dbi_info) = @_;
-    my %extra_info;
+# A hack so that qualify_objects can be tested on SQLite, SQLite does not
+# actually have schemas.
+{
+    sub _table_as_sql {
+        my $self = shift;
+        local $self->{db_schema};
+        return $self->next::method(@_);
+    }
+
+    sub _table_pk_info {
+        my $self = shift;
+        local $self->{db_schema};
+        return $self->next::method(@_);
+    }
+}
+
+sub _columns_info_for {
+    my $self = shift;
+    my ($table) = @_;
+
+    my $result = $self->next::method(@_);
 
     my $dbh = $self->schema->storage->dbh;
-    my $has_autoinc = eval {
-      my $get_seq = $self->{_cache}{sqlite_sequence}
-        ||= $dbh->prepare(q{SELECT count(*) FROM sqlite_sequence WHERE name = ?});
-      $get_seq->execute($table);
-      my ($ret) = $get_seq->fetchrow_array;
-      $get_seq->finish;
-      $ret;
-    };
+    local $dbh->{FetchHashKeyName} = 'NAME_lc';
 
-    if (!$@ && $has_autoinc) {
-        my $sth = $dbh->prepare(
-            "pragma table_info(" . $dbh->quote_identifier($table) . ")"
-        );
-        $sth->execute;
-        my $cols = $sth->fetchall_hashref('name');
-        if ($cols->{$col_name}{pk}) {
-            $extra_info{is_auto_increment} = 1;
+    my $sth = $dbh->prepare(
+      "pragma table_info(" . $dbh->quote_identifier($table) . ")"
+    );
+    $sth->execute;
+    my $cols = $sth->fetchall_hashref('name');
+
+    while (my ($col_name, $info) = each %$result) {
+      if ($cols->{$col_name}{pk} && lc($cols->{$col_name}{type}) eq 'integer') {
+        $info->{is_auto_increment} = 1;
+      }
+    }
+
+    while (my ($col, $info) = each %$result) {
+        if ((eval { ${ $info->{default_value} } }||'') eq 'CURRENT_TIMESTAMP') {
+            ${ $info->{default_value} } = 'current_timestamp';
         }
     }
 
-    return \%extra_info;
+    return $result;
 }
 
 sub _table_fk_info {
@@ -91,14 +109,14 @@ sub _table_fk_info {
         my $rel = $rels[ $fk->{id} ] ||= {
             local_columns => [],
             remote_columns => undef,
-            remote_table => lc $fk->{table}
+            remote_table => $fk->{table}
         };
 
-        push @{ $rel->{local_columns} }, lc $fk->{from};
-        push @{ $rel->{remote_columns} }, lc $fk->{to} if defined $fk->{to};
+        push @{ $rel->{local_columns} }, $self->_lc($fk->{from});
+        push @{ $rel->{remote_columns} }, $self->_lc($fk->{to}) if defined $fk->{to};
         warn "This is supposed to be the same rel but remote_table changed from ",
             $rel->{remote_table}, " to ", $fk->{table}
-            if $rel->{remote_table} ne lc $fk->{table};
+            if $rel->{remote_table} ne $fk->{table};
     }
     $sth->finish;
     return \@rels;
@@ -122,7 +140,7 @@ sub _table_uniq_info {
         $get_idx_sth->execute;
         my @cols;
         while (my $idx_row = $get_idx_sth->fetchrow_hashref) {
-            push @cols, lc $idx_row->{name};
+            push @cols, $self->_lc($idx_row->{name});
         }
         $get_idx_sth->finish;
         push @uniqs, [ $name => \@cols ];
